#========================================================================
# Allow for some system testing using valgrin if available
#========================================================================
VALGRIND_TESTS_ENVIRONMENT ?= $(TESTS_ENVIRONMENT)
VALGRIND_TESTS ?= $(TESTS)
VALGRIND_ALL_OPTS=
VALGRIND_ALL_OPTS+=-v
VALGRIND_ALL_OPTS+=--demangle=yes
VALGRIND_ALL_OPTS+=--suppressions=$(ldas_top_srcdir)/build/data/ldas.supp
VALGRIND_ALL_OPTS+=--suppressions=$(ldas_top_srcdir)/build/data/ospace.supp
VALGRIND_ALL_OPTS+=--suppressions=$(ldas_top_srcdir)/build/data/gcc.supp
VALGRIND_ALL_OPTS+=--num-callers=40
VALGRIND_ALL_OPTS+=$(VALGRIND_TOOL_OPTS)
VALGRIND_ALL_OPTS+=$(VALGRIND_OPTS)

GREP_MEMORY='(definitely lost:|possibly lost:|still reachable:|ERROR)'

CALLGRIND_ALL_OPTS=
CALLGRIND_ALL_OPTS+=-v
CALLGRIND_ALL_OPTS+=--dump-every-bb=10000000

ifdef TESTS
valgrind-gen: check-TESTS
endif

valgrind-clean:
	@if test -d .libs; \
	 then               \
           ( cd .libs; rm -f *.memcheck *.addrcheck ); \
	 fi

memcheck: valgrind-memcheck

valgrind-memcheck:
	@$(MAKE) \
	  VALGRIND_TOOL="memcheck" \
	  VALGRIND_TOOL_OPTS="--leak-check=yes --show-reachable=yes" \
	  valgrind-gen
	@find ./ -name '*.memcheck' \
	  -exec egrep $(GREP_MEMORY) \
	  {} /dev/null \;

valgrind-memcheck-gen-supp:
	@$(MAKE) \
	  VALGRIND_TOOL="memcheck" \
	  VALGRIND_TOOL_OPTS="--leak-check=yes --show-reachable=yes --gen-suppressions=all" \
	  valgrind
	@find ./ -name '*.memcheck' \
	  -exec egrep $(GREP_MEMORY) \
	  {} /dev/null \;

valgrind-addrcheck:
	$(MAKE) \
	  VALGRIND_TOOL="addrcheck" \
	  VALGRIND_TOOL_OPTS="--leak-check=yes --show-reachable=yes" \
	  valgrind
	find ./ -name '*.addrcheck' \
	  -exec egrep '(ERROR SUMMARY:|definitely lost:|possibly lost:|still reachable:|ERROR)' \
	  {} /dev/null \;

valgrind-callgrind:
	$(MAKE) \
	  VALGRIND_TOOL="callgrind" \
	  VALGRIND_TOOL_OPTS="--callgrind-out-file=%q{cmd}.profile.%p" \
	  valgrind-gen

valgrind-helgrind:
	$(MAKE) \
	  VALGRIND_TOOL="helgrind" \
	  VALGRIND_TOOL_OPTS="" \
	  valgrind-gen
	@find ./ -name '*.helgrind' \
	  -exec egrep '(ERROR SUMMARY:|definitely lost:|possibly lost:|still reachable:|ERROR)' \
	  {} /dev/null \;

valgrind-gen: $(VALGRIND_TESTS)
	@for x in $(VALGRIND_TESTS);				\
	do							\
	  if test -x .libs/lt-$$x;				\
	  then							\
	    cmd=".libs/lt-$$x";					\
	  elif test -x .libs/$x;				\
	  then							\
	    cmd=".libs/$$x";					\
	  else							\
	    cmd=".libs/$$x";					\
	  fi;							\
	  export cmd; 						\
	  $(VALGRIND_TESTS_ENVIRONMENT)				\
	  env LD_LIBRARY_PATH="$(VALGRIND_LD_LIBRARY_PATH):$${LD_LIBRARY_PATH}" \
	    $(VALGRIND) --tool=$(VALGRIND_TOOL)			\
	    $(VALGRIND_ALL_OPTS)				\
            $$cmd > $$cmd.$(VALGRIND_TOOL) 2>&1;		\
	done

callgrind: $(VALLGRIND_TESTS)
	echo $(VALGRIND_LD_LIBRARY_PATH)
	for x in $(VALGRIND_TESTS);				\
	do							\
	  if test -x .libs/lt-$$x;				\
	  then							\
	    cmd=".libs/lt-$$x";					\
	  elif test -x .libs/$x;				\
	  then							\
	    cmd=".libs/$$x";					\
	  else							\
	    cmd=".libs/$$x";					\
	  fi;							\
	  $(VALGRIND_TESTS_ENVIRONMENT)				\
	  env LD_LIBRARY_PATH="$(VALGRIND_LD_LIBRARY_PATH):$${LD_LIBRARY_PATH}" \
	  callgrind $(CALLGRIND_ALL_OPTS) --base=$$cmd $$cmd;	\
	done

clean: valgrind-clean
