#ifndef FrameCPP_VERSION_8_FrStatData_HH
#define FrameCPP_VERSION_8_FrStatData_HH

#include <vector>

#include "framecpp/Common/TOCInfo.hh"
#include "framecpp/Common/SearchContainer.hh"
#include "framecpp/Common/FrameStream.hh"
#include "framecpp/Common/FrStatData.hh"

#include "framecpp/Version7/FrStatData.hh"

#include "framecpp/Version8/FrameSpec.hh"
#include "framecpp/Version8/STRING.hh"
#include "framecpp/Version8/PTR_STRUCT.hh"

#include "framecpp/Version8/FrDetector.hh"
#include "framecpp/Version8/FrSE.hh"
#include "framecpp/Version8/FrTable.hh"
#include "framecpp/Version8/FrTOC.hh"
#include "framecpp/Version8/FrVect.hh"

namespace FrameCPP
{
  namespace Version_8
  {
    //===================================================================
    //===================================================================

    typedef Previous::FrStatData::FrStatDataNPS FrStatDataNPS;

    //===================================================================
    //===================================================================

    typedef Version_4::FrStatDataPST< PTR_STRUCT,
				      FrDetector, FrVect, FrTable>
    FrStatDataPS;

    //===================================================================
    //===================================================================
    class FrStatData
      : public ObjectWithChecksum< Common::FrStatData >,
	public FrStatDataNPS,
	public FrStatDataPS,
	public Common::TOCInfo
    {
    public:
      //: Default constructor
      FrStatData( );
      FrStatData( const FrStatData& statData );
      FrStatData( const std::string& name, const std::string& comment,
		  const std::string& representation,
		  INT_4U timeStart,INT_4U timeEnd,
		  INT_4U version );
      FrStatData( const Previous::FrStatData& Source,
		  Common::IStream* Stream );

      static std::auto_ptr< Object >
      Promote( INT_2U Source,
	       Object* Obj, Common::IStream* Stream );

      static const char* const StructName( );

      static const Common::Description* StructDescription( );

      virtual FrStatData* Clone( ) const;

      virtual const FrDetector* GetDetector( ) const
      {
	return FrStatDataPS::GetDetector( );
      }

      virtual const char* const ObjectStructName( ) const;

      virtual void TOCQuery( int InfoClass, ... ) const;

      //: Merge with another FrAdcData
      FrStatData& Merge( const FrStatData& RHS );

      /// \brief comparison operator
      bool operator==( const FrStatData& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

    protected:
      //-----------------------------------------------------------------
      /// \brief Down grade an object
      //-----------------------------------------------------------------
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Upgrade an object
      //-----------------------------------------------------------------
      virtual std::auto_ptr< Object >
      promote( INT_2U Target, Object* Obj, Common::IStream* Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Number of bytes needed to write this structure
      //-----------------------------------------------------------------
      cmn_streamsize_type pBytes( const Common::StreamBase& Stream) const;

      //-----------------------------------------------------------------
      /// \brief Number of bytes needed to write this structure
      //-----------------------------------------------------------------
      virtual FrStatData* pCreate( Common::IStream& Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Write the structure to the stream
      //-----------------------------------------------------------------
      virtual void pWrite( Common::OStream& Stream ) const;

    private:
      //-----------------------------------------------------------------
      //-----------------------------------------------------------------
      typedef ObjectWithChecksum< Common::FrStatData > object_type;

      using Common::FrameSpec::Object::Create;

      FrStatData( Common::IStream& Stream );
    };

    //===================================================================
    //===================================================================
    inline std::auto_ptr< Common::FrameSpec::Object > FrStatData::
    Promote( INT_2U Source,
	     Common::FrameSpec::Object* Obj, Common::IStream* Stream )
    {
      return Object::PromoteObject< Previous::FrStatData, FrStatData >
	( DATA_FORMAT_VERSION, Source, Obj, Stream );
    }

    inline const char* const FrStatData::
    StructName( )
    {
      static const char* const name = "FrStatData";
      return name;
    }
  } // namespace - Version_8
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_8_FrStatData_HH */
